package it.mokabyte.cache;

import it.mokabyte.cache.exception.NotFoundObjectException;
import it.mokabyte.cache.key.KeyObjectId;

/**
 *  L'interfaccia descrive il generico componente di Cache
 *
 *@author     lucadozio
 *@created    19 maggio 2003
 */
public interface ICache {
    /**
     *  Acquisisco l'oggetto salvato nella cache con una propria chiave
     *
     *@param  key                          chiave di memorizzazione in cache
     *@return                              il generico oggetto salvato in cache
     *@exception  NotFoundObjectException  indica che l'oggetto non  stato trovato in cache
     */
    public Object get(KeyObjectId key) throws NotFoundObjectException;


    /**
     *  Inserimento dell'oggetto in cache
     *
     *@param  key         chiave di memorizzazione
     *@param  objToCache  oggetto da inserire nella cache
     */
    public void put(KeyObjectId key, Object objToCache);


    /**
     *  Rimozione dell'oggetto dalla cache
     *
     *@param  key                          identificativo dell'oggetto in cache
     *@exception  NotFoundObjectException  indica che l'oggetto non  stato trovato
     */
    public void remove(KeyObjectId key) throws NotFoundObjectException;


}
