package it.mokabyte.cache;

import it.mokabyte.cache.exception.NotFoundObjectException;
import it.mokabyte.cache.key.KeyObjectId;
import java.util.Hashtable;

/**
 *  implementazione di una classe locale
 *
 *@author     lucadozio
 *@created    26 maggio 2003
 */
public class LocalCache implements ICache {

    /**
     *  supporto che realizza la Cache in locale, che  una copia della cache
     *  distribuita sul supporo JNDI
     */

    //L'uso di Hashtable risparmia l'uso di synchronized per accedere all'istanza.
    private static Hashtable localCache = null;


    /**
     *  Constructor for the LocalCache object
     */
    public LocalCache() {
        if (localCache == null) {
            localCache = new Hashtable();
        }
    }


    /**
     *  Il metodo getObjectInCache carica l'oggeto in cache cercando prima in
     *  locale e poi in distribuito
     *
     *@param  key                          chiave per il recupero dell'oggetto
     *@return                              Object in cache restituito
     *@exception  NotFoundObjectException  Description of the Exception
     *@throws  NotFoundObject              eccezzione generata nel caso non si
     *      torvi l'oggetto in cache
     */
    public Object get(KeyObjectId key) throws NotFoundObjectException {
        Object retObj = null;
        if (!isObjectInCache(key)) {
            throw new NotFoundObjectException("L'oggetto [" + key.getId() + "] di classe [" + key.getInfoClassKey() + "] non  in cache");
        } else {
            synchronized (localCache) {
                retObj = localCache.get(key);
            }
        }
        return retObj;
    }


    /**
     *  Il metodo addObject, aggiunge alla cache locale e remota l'oggetto
     *  recuperato dal flusso di esecuzione
     *
     *@param  key          la chiave per la registrazione dell'oggetto nella
     *      cache
     *@param  objToCache  oggetto da porre in cache
     */
    public void put(KeyObjectId key, Object objToCache) {
        this.putLocalObject(key, objToCache);
    }


    /**
     *  rimozione dell'oggetto dalla cache
     *
     *@param  key                          chiave per la rimozione dell'oggetto
     *@exception  NotFoundObjectException  l'oggetto non  presente nella cache
     */
    public void remove(KeyObjectId key) throws NotFoundObjectException {
        if (!isObjectInCache(key)) {
            throw new NotFoundObjectException("L'oggetto [" + key.getId() + "] di classe [" + key.getInfoClassKey() + "] non  in cache");
        }
        removeLocal(key);
    }


    /**
     *  Gets the objectInCache attribute of the LocalCache object
     *
     *@param  key  Description of the Parameter
     *@return      The objectInCache value
     */
    private boolean isObjectInCache(Object key) {
        return localCache.containsKey(key);
    }


    /**
     *  Description of the Method
     *
     *@param  key                          Description of the Parameter
     *@exception  NotFoundObjectException  Description of the Exception
     */
    private void removeLocal(KeyObjectId key) throws NotFoundObjectException {
        System.out.println("rimozione dalla cache locale.." + key);
        if (isObjectInCache(key)) {
            System.out.println("siamo nel remove action di ..." + key);
            this.localCache.remove(key);
        } else {
            throw new NotFoundObjectException("L'oggetto [" + key.getId() + "] di classe [" + key.getInfoClassKey() + "] non  in cache");
        }

    }


    /**
     *  Description of the Method
     *
     *@param  key         Description of the Parameter
     *@param  objToCache  Description of the Parameter
     */
    private void putLocalObject(KeyObjectId key, Object objToCache) {
        System.out.println("inserito in locale....[" + key + "]---[" + objToCache + "]");
        localCache.put(key, objToCache);
    }


    /**
     *  reperimento dell'insieme di chiavi in cache
     *
     *@return    i valori dell'insieme di chiavi
     */
    public java.util.Set getKeySet() {
        return localCache.keySet();
    }


    /**
     *  rimozione di tutti gli oggetti in cache
     */
    public void removeAll() {
        localCache.clear();
    }


}
