package it.mokabyte.cache.key;

import java.lang.reflect.Method;
import java.util.HashMap;

/**
 * Classe che realizza l'algoritmo di creazione della chiave per la cache
 *
 * @author lucadozio
 * @version 1.0
 * @see java.lang.reflect.Field
 * @see java.lang.reflect.Method
 * @see java.util.HashMap
 *
 */

public class KeyCreator {

	/**
	 * istanza univoca di singleton pattern
	 */
	private static KeyCreator oneInstance = null;

	/**
	 * istanza unica di keyRepository
	 */
	private static HashMap keyRepository = null;

	/**
	 * Constructor privato per il singleton pattern
	 */
	private KeyCreator() {
		super();
	}

	/**
	 * costruzione dell'unica istanza di oggetto
	 */
	public static KeyCreator getInstance(){
		synchronized(KeyCreator.class){
			if (oneInstance ==null){
				oneInstance=new KeyCreator();
				keyRepository = new HashMap();
			}
			return oneInstance;
		}
	}

	/**
	 * recupero dell'oggetto chiave, per la memorizzazioen in cache
	 * @param requestToPerform oggetto che viene utilizzato per la costruzione della chiave
	 * @return classe di identificativo univoco (KeyObjectId)
	 */
	public final static synchronized KeyObjectId getKey(Object requestToPerform){
		KeyObjectId retKey = null;
		String perfResult = generateId(requestToPerform);
		if(!keyRepository.containsKey(perfResult)){
			retKey = new KeyObjectId(perfResult,requestToPerform);
			keyRepository.put(perfResult,retKey);
		}else{
			retKey = (KeyObjectId) keyRepository.get(perfResult);
		}
		return retKey;
	}


	/**
	 * metodo che implementa l'algoritmo per la generazione della chiave di registrazione
	 * @param requestToPerform oggetto che subisce l'elaborazione
	 * @return String che  il JDNIName per la registrazione dell'oggetto in cache distribuita.
	 */
	protected static synchronized String generateId (Object requestToPerform){
		//algoritmo per l'elaborazione della chiave
		System.out.println("-->.......");
		if (requestToPerform instanceof java.lang.String){
                  return (String)requestToPerform;
                }
		//algoritmo di generazione key:
		StringBuffer sbKey = new StringBuffer(requestToPerform.getClass().getName());
		sbKey.append("-");
		java.lang.reflect.Field[] fields = requestToPerform.getClass().getDeclaredFields();
		for (int i=0; i<fields.length; i++){
			String fieldName = fields[i].getName();
			Class  filedType = fields[i].getType();
			System.out.println("fn-->"+ fieldName + " ft-->"+ filedType);
			sbKey.append(fieldName).append("_");
			String getFieldName = getMaker(fieldName);
			try{
				Method m = requestToPerform.getClass().getMethod(getFieldName,null);
				Object retInvoke = m.invoke(requestToPerform,null);
				System.out.println("--->>" + retInvoke);
				if(retInvoke instanceof java.util.ArrayList)
					sbKey.append(((java.util.ArrayList)retInvoke).size());
				else
					sbKey.append(retInvoke);
			}catch(Exception e){
				//e.printStackTrace();
				sbKey.append("null");
			}
			if (i != (fields.length-1)) sbKey.append("-");

		}


		return sbKey.toString();
	}


	/**
	 * metodo che realizza la costruzione del nome degli "accessors methods"
	 * @return geAttribute
	 */
	private static String getMaker(String attribute){
		String first = attribute.substring(0,1);
		StringBuffer sb = new StringBuffer(attribute);
		sb.replace(0,1,first.toUpperCase());
		return "get"+sb.toString();

	}





}