package it.mokabyte.cache.key;

import java.io.Serializable;

/**
 * Classe di identificativo univoco dell'oggetto registrato in cache
 * 
 * @author lucadozio
 * @version 1.0
 * @see java.io.Serializable
 * 
 */
public class KeyObjectId implements Serializable {

	/**
	 * identificativo univoco di classe in cache
	 */
	private String id = null;	
	/**
	 * informazioni sulla classe da cui si  generata la chiave
	 */
	private String infoClassKey = null;
	/**
	 * Costruttore con la stringa di chiave remota.
	 */
	public KeyObjectId(String keyId) {
		this.id=keyId;
	}
	/**
	 * Costruttore con la stringa di chiave remota, e l'informazione sulla classe in chiave
	 * @param keyId stringa di chaive remota
	 * @param requestToPerform oggetto da cui  tratta la chiave
	 */
	public KeyObjectId(String keyId, Object requestToPerform) {
		this.id=keyId;
		this.infoClassKey=requestToPerform.getClass().getName();
	}

	/**
	 * Ritorno dell'identificativo di oggetto in cache
	 * @return String
	 */
	public String getId() {
		return id;
	}

	/**
	 * Set dell'identificativo univoco di oggetto in cache.
	 * @param id univoco dell'oggetto in cache (JNDI name)
	 */
	public void setId(String id) {
		this.id = id;
	}
	/**
	 * Ritorna le informazioni sulla classe che genera la chiave
	 * @return String di informazioni sulla classe in chiave
	 * 
	 */
	public String getInfoClassKey() {
		return infoClassKey;
	}

	/**
	 * Set dell'informazioni sulla classe in chiave 
	 * @param infoClassKey 
	 */
	public void setInfoClassKey(String infoClassKey) {
		this.infoClassKey = infoClassKey;
	}
	
	/**
	 * metodo per la verifica dell'ugualgianza di due oggetti
	 * @param oggetto con cui fare il confronto
	 */
	public boolean equals(Object o){
		//System.out.println("invoked equals....");
		boolean ret = false;
		if (o instanceof KeyObjectId){
			KeyObjectId key = (KeyObjectId)o;
			if((key.getId()==this.id) && 
				(key.getInfoClassKey()==this.infoClassKey)) ret=true;
		}
		return ret;
	}
	
	/**
	 * metodo che ritona le informazioni sulla classe
	 * @return stringa con le informazioni sul valore dei campi della classe
	 */
	public String toString(){
		StringBuffer sb = new StringBuffer(getClass().getName());
		sb.append(">>keyId:").append(id);
		sb.append(">>infoClassKey:").append(infoClassKey);
		return sb.toString();
	}

}
