package it.mokabyte.cache.policy;

import it.mokabyte.cache.locator.CacheServiceLocator;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.jms.*;
import javax.naming.*;
import javax.swing.*;

/**
 *  Applicazione JMS Publisher/Subscriber per la gestioen delle policy di cache
 *
 *@author     lucadozio
 *@created    26 maggio 2003
 */
public class PolicyEventGenerator {

    /*
     *  Componenti interfaccia grafica
     */
    private JFrame frame = null;
    private JButton bClear = null;
    private JButton bSend = null;
    private JTextField tfTx = null;
    private JTextArea taRx = null;

    /**
     *  Il nome dell'utente
     */
    private String userName = null;
    /**
     *  La connesione JMS
     */
    private TopicConnection topicConnection = null;
    /**
     *  Il Message sender
     */
    private TopicPublisher topicPublisher = null;
    /**
     *  Il messaggio
     */
    private TextMessage message = null;


    /**
     *@param  args  parametri da riga di comando
     */
    public static void main(String[] args) {

        if (args.length == 0) {
            new PolicyEventGenerator("MokaTopicConnectionFactory", "Luke");
        } else if ((args.length == 1)) {
            new PolicyEventGenerator(args[0], "Luke");
        } else if ((args.length == 2)) {
            new PolicyEventGenerator(args[0], args[1]);
        } else {
            System.out.println("Usage:[<topic-connection-factory><topic-name>]");

        }
    }


    /**
     *  Costruttore
     *
     *@param  topicConnectionFactoryName  Connection Factory name
     *@param  topicName                   Destination name
     */
    public PolicyEventGenerator(String topicConnectionFactoryName, String topicName) {
        try {
            Context jndiContext = CacheServiceLocator.getContext();
            System.out.println("JNDI Context[" + jndiContext + "]...");

            // Look up connection ConnectionFactory e detination Topic
            System.out.println("Looking up connection factory [" + topicConnectionFactoryName + "]...");
            TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) jndiContext.lookup(topicConnectionFactoryName);

            System.out.println("Looking up topic [" + topicName + "]...");
            Topic topic = (Topic) jndiContext.lookup("topic/" + topicName);

            System.out.println("topicConnectionFactory [" + topicConnectionFactory + "] - topic [" + topic + "]");
            this.go(topicConnectionFactory, topic);

        } catch (NamingException ne) {
            System.out.println(" # NamingException : " + ne.toString());
            ne.printStackTrace();
            System.exit(1);
        }
    }


    /**
     *  Configurazione dell'applicazione (publisher e subscriber del Topic)
     *
     *@param  topicConnectionFactory  l'oggetto Connection Factory
     *@param  topic                   l'oggetto destinazione Topic
     */
    private void go(TopicConnectionFactory topicConnectionFactory, Topic topic) {

        try {
            this.readUserName();
            //legge il nome dell'utente da stdin

            this.buildGUI();
            //costruisce la GUI

            // JMS
            this.topicConnection = topicConnectionFactory.createTopicConnection();
            System.out.println("topicConnection [" + topicConnection + "]");

            TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            System.out.println("topicSession [" + topicSession + "]");

            // Si crea l'oggetto message
            this.message = topicSession.createTextMessage();

            // Parte di publisher
            this.topicPublisher = topicSession.createPublisher(topic);
            System.out.println("topicPublisher [" + topicPublisher + "]");

             System.out.println("Inizializzazione JMS completata ...");

        } catch (JMSException e) {
            System.out.println(" # JMSException : " + e.toString());
            e.printStackTrace();
            System.exit(0);
        }
    }


    /**
     *  Legge da tastiera il nome dell'utente
     */
    private void readUserName() {
        try {
            BufferedReader msgStream = new BufferedReader(new InputStreamReader(System.in));
            System.out.print("Inserisci il tuo nome : ");
            this.userName = msgStream.readLine();
        } catch (IOException ioe) {
            this.userName = "";
        }
    }


    /**
     *  Costruisce l'interfaccia grafica
     */
    public void buildGUI() {
        // Costruzione del Frame
        frame = new JFrame("Policy Manager : utente[" + userName + "]");
        frame.getContentPane().setLayout(new BorderLayout());

        // inizializzazione degli oggetti della GUI
        tfTx = new JTextField();
        taRx = new JTextArea();
        taRx.setEditable(false);
        bClear = new JButton("CLEAR");
        bSend = new JButton("SEND");

        // disposizione dei componenti grafici
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(bClear, BorderLayout.WEST);
        p.add(tfTx, BorderLayout.CENTER);
        p.add(bSend, BorderLayout.EAST);

        // il bottone send invia il messaggio contenuto nel TextField
        bSend.addActionListener(
            new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        message.setText(tfTx.getText());
                        topicPublisher.publish(message);
                        System.out.println("Message[" + tfTx.getText() + "] sent ...");
                    } catch (JMSException jmse) {
                        tfTx.setText(jmse.getMessage());
                        jmse.printStackTrace();
                    }
                }
            });

        // il bottone clear ripulisce il TextField
        bClear.addActionListener(
            new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    tfTx.setText("");
                }
            });

        // l'evento di chiusura della finestra genera la chiusura
        // della connessione JMS
        frame.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.out.println("windowClosing : JMS clean-up code...");
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException jmse) {
                        }
                    }
                    System.exit(0);
                }
            });

        frame.getContentPane().add(p, BorderLayout.NORTH);
        JScrollPane sc = new JScrollPane(taRx);
        frame.getContentPane().add(sc, BorderLayout.CENTER);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setSize(screenSize.width / 2, screenSize.height / 20);
        Dimension frameSize = frame.getSize();
        frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
				frame.setVisible(true);
    }
}
