package it.mokabyte.pattern.controller.handler;

import javax.servlet.http.*;
import javax.servlet.*;

import it.mokabyte.pattern.controller.exception.ServiceException;

import it.mokabyte.pattern.valueobject.AccountOM;

import it.mokabyte.cache.manager.CacheManager;
import it.mokabyte.cache.exception.NotFoundObjectException;

import it.mokabyte.pattern.view.AccountViewBean;
import it.mokabyte.pattern.view.ErrorViewBean;

import it.mokabyte.pattern.delegate.MvcBusinessDelegate;
import it.mokabyte.pattern.delegate.MvcFactoryBusinessDelegate;
import it.mokabyte.pattern.delegate.MvcBusinessDelegateImpl;

import it.mokabyte.pattern.exception.BusinessDelegateException;

/**
 * <p>Title: BaseRequestHandler</p>
 * <p>Description: classe handler per la funzione di visulaizzazione dell'account con Cache</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author S.Rossini, Luca Dozio
 * @version 1.1
 */
public class AccountRequestHandler extends BaseRequestHandler {

 public void execute(HttpServletRequest request, HttpServletResponse response, ServletContext context) throws ServiceException{

    MvcBusinessDelegate proxy = null;

    String userid = request.getParameter("USER_ID");
    System.out.println("AccountRequestHandler.execute : USER_ID[" + userid+"]...");
    System.out.println("AccountRequestHandler.execute : session["+session+"]...");

    // -----------------------------------------------------------------------------------------
    // Se si volesse memorizzare in sessione il BusinessDelegate, scommentare il seguente codice ...
    /*
    if(this.session != null) {
      proxy = (MvcBusinessDelegate)session.getAttribute("BUSINESS_DELEGATE");
      System.out.println("AccountRequestHandler.execute : proxy["+proxy+"]...");
      if(proxy == null) {
          System.out.println("AccountRequestHandler.execute : creating Business Delegate ...");
          proxy = MvcFactoryBusinessDelegate.getBusinessDelegate(this.getClass().getName(),userid);
          this.session.setAttribute("BUSINESS_DELEGATE",proxy);
       }
    }
    */

    try {
      //parte modificata per la cache 
        AccountOM model = null;
        CacheManager chm = new  CacheManager();
        try{
          System.out.println("##########################TRY WITH CACHE          ################################");
          model= (AccountOM)chm.fetchObject(userid);
          System.out.println("##########################EXECUTED WITH CACHE     ################################");
        }catch(NotFoundObjectException nfoe){

          System.out.println("##########################EXECUTED WITHOUT CACHE  #############################");

          proxy = MvcFactoryBusinessDelegate.getBusinessDelegate(this.getClass().getName(),userid);

          System.out.println("AccountRequestHandler.execute : proxy class ["+proxy.getClass().getName()+"]...");

          model = ((MvcBusinessDelegateImpl)proxy).getAccount();
          chm.putObject(userid,model);
          System.out.println("AccountRequestHandler.execute : model["+model+"]...");
       }
       //fine parte modificata per la cache 
       AccountViewBean view = new AccountViewBean(model);
       System.out.println("AccountRequestHandler.execute : view["+view+"]...");

       request.setAttribute("userAccount", view);

       // ok next page
       this.pageName = "/jsp/displayAccount.jsp";

    }catch(BusinessDelegateException bde) {

       // error next page
       this.pageName = "/jsp/error/GetAccountError.jsp";

       // COSTRUIRE ERROR VIEW BEAN efalloleggere allapagina JSP !
       ErrorViewBean errorViewBean = new ErrorViewBean(bde,"Errrore durante la fase di lettura Account ["+userid+"]");
       request.setAttribute("errore",errorViewBean);
       System.out.println("# EXCEPTION on AccountHandler : " + bde.getMessage());
       throw new ServiceException(" # EXCEPTION on AccountHandler : " + bde.getMessage());
    }
 }

}